/*
 * Created on 09/06/2004
 *
 */
package logic.signedFormulas;

import logic.formulas.FormulaFactory;
import logic.problem.Problem;
import parsers.ParserUser;

/**
 * Class that allows the creation of formulas from strings (using Wagner Dias's
 * syntax). Ex.: FormulaCreator fc = new FormulaCreator(); fc.parseString ("T
 * A-> B");
 */
public class SignedFormulaCreator {

    // TODO use Format as superclass

    SignedFormulaFactory _sff;

    FormulaFactory _ff;

    Problem _problem;

    String jar1, lexerClassName1, parserClassName1, jar2, lexerClassName2,
            parserClassName2, _packageName;

    String _libdir = "file:" + System.getProperty("user.home")
            + "/MSTP/lib/generated/";

    boolean _twoPhases = true;

    public SignedFormulaCreator() {
        _sff = new SignedFormulaFactory();
        _ff = new FormulaFactory();
        _packageName = "sats2";
    }

    public SignedFormulaCreator(String packageName) {
        _sff = new SignedFormulaFactory();
        _ff = new FormulaFactory();
        _packageName = packageName;
    }

    public SignedFormulaCreator(String packageName, String libDir,
            boolean twoPhases) {
        this(packageName);
        setLibDir(libDir);
        setTwoPhases(twoPhases);
    }

    public void setLibDir(String libDir) {
        _libdir = "file:" + libDir;
    }

    public void setTwoPhases(boolean option) {
        _twoPhases = option;
    }

    public Problem parseText(String signedFormulasAsString) {
        String s;
        if (_twoPhases) {
            ParserUser pu1 = new ParserUser(new String[] { _libdir + "cw.jar" });

            s = (String) pu1.parseString(
                    "ConversorWagnerSATLIB.ConversorWagnerSATLIBLexer",
                    "ConversorWagnerSATLIB.ConversorWagnerSATLIBParser",
                    signedFormulasAsString);
        } else {
            s = signedFormulasAsString;
        }

        ParserUser pu2 = new ParserUser(new String[] { _libdir + _packageName
                + ".jar" });
        _problem = (Problem) pu2.parseString(_packageName + "." + _packageName
                + "Lexer", _packageName + "." + _packageName + "Parser", s);

        _sff.cloneAll(_problem.getSignedFormulaFactory(), _ff);

        return _problem;
    }

    /**
     * Converts a string to a signed formula.
     * 
     * @param signedFormulaAsString
     * @return
     */
    public SignedFormula parseString(String signedFormulaAsString) {
        
        this.parseText(signedFormulaAsString);

        return (SignedFormula) _sff.getSignedFormulas().get(
                _problem.getSignedFormulaFactory().getLastSignedFormulaAdded()
                        .toString());
    }

    public Problem parseFile(String completeFilename) {

        String s;
        if (_twoPhases) {
            ParserUser pu1 = new ParserUser(new String[] { _libdir + "cw.jar" });

            s = (String) pu1.parseFile(
                    "ConversorWagnerSATLIB.ConversorWagnerSATLIBLexer",
                    "ConversorWagnerSATLIB.ConversorWagnerSATLIBParser",
                    completeFilename);

            ParserUser pu2 = new ParserUser(new String[] { _libdir
                    + _packageName + ".jar" });
            _problem = (Problem) pu2.parseString(_packageName + "."
                    + _packageName + "Lexer", _packageName + "." + _packageName
                    + "Parser", s);

            _sff.cloneAll(_problem.getSignedFormulaFactory(), _ff);
        } else {
            ParserUser pu2 = new ParserUser(new String[] { _libdir
                    + _packageName + ".jar" });
            _problem = (Problem) pu2.parseFile(_packageName + "."
                    + _packageName + "Lexer", _packageName + "." + _packageName
                    + "Parser", completeFilename);

            _sff.cloneAll(_problem.getSignedFormulaFactory(), _ff);
        }

        return _problem;

    }

    public SignedFormulaFactory getSignedFormulaFactory() {
        return _sff;
    }

    public FormulaFactory getFormulaFactory() {
        return _ff;
    }

}