/*
 * Created on 07/04/2005
 *
 */
package logicalSystems.mbc;

import logic.formulas.Connective;
import logic.logicalSystem.ISignature;
import logic.signedFormulas.FormulaSign;
import rules.Rule;
import rules.structures.ConnectiveRoleSignRuleList;
import rules.structures.ConnectiveRuleStructureFactory;
import rules.structures.OnePremiseNoConnectiveRuleList;
import rules.structures.OnePremiseRuleList;
import rules.structures.PBRuleList;
import rules.structures.RuleList;
import rules.structures.RuleType;
import rules.structures.RulesStructure;

/**
 * @author Adolfo Gustavo Serra Seca Neto
 * 
 *  
 */
public class MBCRuleStructures {

    private ISignature signature;

    private OnePremiseRuleList onePremiseRules;

    private OnePremiseNoConnectiveRuleList optionalOnePremiseRules;

    private ConnectiveRoleSignRuleList twoPremiseRules;

    private ConnectiveRoleSignRuleList optionalTwoPremiseRules;

    private PBRuleList PBRules;

    private PBRuleList optionalPBRules;

    private RulesStructure _rules;

    private ConnectiveRuleStructureFactory crsf = new ConnectiveRuleStructureFactory();

    // constants for lists of rules
    private static final String ONE_PREMISE_RULES = "opr";

    private static final String TWO_PREMISE_RULES = "tpr";

    private static final String OPTIONAL_TWO_PREMISE_RULES = "otpr";

    private static final String OPTIONAL_ONE_PREMISE_RULES = "oopr";

    private static final String PB_RULES = "pbr";

    private static final String OPTIONAL_PB_RULES = "opbr";

    public MBCRuleStructures(ISignature signature) {
        this.signature = signature;

        /** mandatory one premise rules */
        onePremiseRules = initializeOnePremiseRuleList();

        /** mandatory two premise rules */
        twoPremiseRules = initializeTwoPremsieRuleList();

        /** optional two premise rules */
        optionalTwoPremiseRules = initializeoOptionalTwoPremiseRuleList();

        /** optional one premise rules */
        optionalOnePremiseRules = initializeOptionalOnePremiseRuleList();

        /** mandatory rules for applying PB */
        PBRules = initializePBRuleList();

        /** optional rules for applying PB */
        optionalPBRules = initializeOptionalPBRuleList();

        /** Order of the sets of rules */
        // the order is important!
        _rules = new RulesStructure();
        _rules.add(ONE_PREMISE_RULES, onePremiseRules);
        _rules.add(TWO_PREMISE_RULES, twoPremiseRules);
        _rules.add(OPTIONAL_TWO_PREMISE_RULES, optionalTwoPremiseRules);
        _rules.add(OPTIONAL_ONE_PREMISE_RULES, optionalOnePremiseRules);
        _rules.add(PB_RULES, PBRules);
        _rules.add(OPTIONAL_PB_RULES, optionalPBRules);
    }

    public RulesStructure getRuleStructure() {
        return _rules;
    }

    public RuleList getRules(Connective conn) {
        return crsf.createCRS(conn).getRules();
    }

    
    /**
     * @return
     */
    private OnePremiseRuleList initializeOnePremiseRuleList() {
        onePremiseRules = new OnePremiseRuleList();

        addToOnePremiseRules(MBCSigns.FALSE, MBCConnectives.NOT,
                MBCRules.F_NOT);
        addToOnePremiseRules(MBCSigns.TRUE, MBCConnectives.AND,
                MBCRules.T_AND);
        addToOnePremiseRules(MBCSigns.FALSE, MBCConnectives.OR,
                MBCRules.F_OR);
        addToOnePremiseRules(MBCSigns.FALSE,
                MBCConnectives.IMPLIES, MBCRules.F_IMPLIES);

        
        return onePremiseRules;
    }

    private void addToOnePremiseRules(FormulaSign sign, Connective conn, Rule r) {
        if (signature.contains(conn)) {
            addConnectiveRuleType(conn, r, RuleType.SIMPLE_1P);
            onePremiseRules.add(sign, conn, r);
        }
    }

    private void addConnectiveRuleType(Connective conn, Rule r, RuleType rt) {
        crsf.createCRS(conn).add(r, rt);
    }


    /**
     * @return
     */
    private ConnectiveRoleSignRuleList initializeTwoPremsieRuleList() {
        // TODO Auto-generated method stub
    	return null;
    }

    /**
     * @return
     */
    private ConnectiveRoleSignRuleList initializeoOptionalTwoPremiseRuleList() {
        // TODO Auto-generated method stub
    	return null;
    }

    /**
     * @return
     */
    private OnePremiseNoConnectiveRuleList initializeOptionalOnePremiseRuleList() {
        optionalOnePremiseRules= new OnePremiseNoConnectiveRuleList();

        optionalOnePremiseRules.add (MBCSigns.FALSE, MBCRules.F_FORMULA);
        
        return optionalOnePremiseRules;
    }

    /**
     * @return
     */
    private PBRuleList initializePBRuleList() {
        // TODO Auto-generated method stub
    	return null;
    }

    /**
     * @return
     */
    private PBRuleList initializeOptionalPBRuleList() {
        // TODO Auto-generated method stub
    	return null;
    }

}
