/*
 * Created on 04/02/2005
 *  
 */
package main.aspects;

import main.proofTree.Node;
import main.proofTree.ProofTree;

/** Aspect that manages information about the level of a node in a proof tree.
 * @author Adolfo Gustavo Serra Seca neto
 *  
 */


/** Every node has a position represented by its branch id (a number) and an order 
 * in this branch (also a number). From this you can derive:
 * the level of the branch, 
 * a unique ordering (not sequential) number for each node,
 * a string describing the branch of the node (for instance: llr means right branch of
 * the left branch of the left branch of the first (root) branch.
 * 
 * 
 * @author Adolfo Gustavo Serra Seca neto
 *
 */
class NodePosition{
    
    
}

class BranchId{
    
    int branchId;
    
    public static final BranchId ROOT = new BranchId(1); 
    public static final int LEFT = 0; 
    public static final int RIGHT = 1; 
    
    BranchId (int id){
        this.branchId = id;
    }

    BranchId (BranchId bid, int position){
        this.branchId = bid.getValue()*2 + position;
    }
    
    public int getValue(){
        return branchId;
    }
    
    public String toString(){
//        return Integer.toString(branchId);
        String result = "";
        int auxBranchId = branchId;
        
        if (auxBranchId == 1){
            return "1 rt";
        }
        while (auxBranchId > 0){
            result = ((auxBranchId %2 ==LEFT)?"l":"r") + result;
            auxBranchId/=2;
            
        }
        return branchId + " " + result.substring(1);
    }
}

public aspect ProofTreeHeightAspect {
    
    // CRIAR CLASSE - SISTEMA TERNRIO
    // l = 1
    // r = 2
    // lr = 12 = 1x3 + 2x3

    // inter-type declarations
    
    private int ProofTree.level;

    private String ProofTree.branchNumber;

    public void ProofTree.setLevel(int l) {
        level = l;
    }

    public int ProofTree.getLevel() {
        return level;
    }

    public void ProofTree.setBranchNumber(String number) {
        branchNumber = number;
    }

    public String ProofTree.getBranchNumber() {
        return branchNumber;
    }

    private BranchId ProofTree.branchId;
    
    public void ProofTree.setBranchId(BranchId id) {
        branchId = id;
    }

    public void ProofTree.setBranchId(BranchId id, int position) {
        branchId = new BranchId(id, position);
    }

    public BranchId ProofTree.getBranchId() {
        return branchId;
    }

    

    // pointcuts for knowing when a node is added to a proof tree
    pointcut ProofTreeRootConstructor(ProofTree pt, Node n):target(pt) && args(n) 
        && execution(public ProofTree.new(Node+))
        && !cflow(execution(* ProofTree+.add*(*)));

    pointcut ProofTreeLeftConstructor(ProofTree pt, Node n):
        target(pt) &&
        args (n) &&
        call(* ProofTree+.addLeft(*))
                ;

    pointcut ProofTreeRightConstructor(ProofTree pt, Node n):
        target(pt) &&
        args (n) &&
        call(* ProofTree+.addRight(*))
                ;

    pointcut ProofTreeAddNode(ProofTree pt, Node n):
        target(pt) &&
        args (n) &&
        call(* ProofTree+.addLast(*))
                ;

    // after a node is added
    after(ProofTree pt, Node n): 
        ProofTreeRootConstructor(pt, n){
//        System.out.println(thisJoinPoint.getSignature());
        pt.setLevel(0);
        pt.setBranchNumber("");
        pt.setBranchId(BranchId.ROOT);
        //        System.out.println(pt.getBranchNumber()+" raiz do nvel " +
        // pt.getLevel() + " :" + n);
        //        System.out.println(pt.getBranchNumber()+ n);
        showNode(pt, n);

    }

    ProofTree around(ProofTree pt, Node n): 
        ProofTreeLeftConstructor(pt, n){
//        System.out.println(thisJoinPoint.getSignature());

        ProofTree left = proceed(pt, n);
        left.setLevel(left.getParent().getLevel() + 1);
        left.setBranchNumber(left.getParent().getBranchNumber() + "l");
        left.setBranchId(left.getParent().getBranchId(), BranchId.LEFT);
        //        System.out.println(left.getBranchNumber()+" primeiro da esquerda do
        // nvel " + left.getLevel()
        //                + " :" + n);
        //        System.out.println(left.getBranchNumber()+ n);
        showNode(left, n);

        return left;

    }

    ProofTree around(ProofTree pt, Node n): 
        ProofTreeRightConstructor(pt, n){
//        System.out.println(thisJoinPoint.getSignature());

        ProofTree right = proceed(pt, n);
        right.setLevel(right.getParent().getLevel() + 1);
        right.setBranchNumber(right.getParent().getBranchNumber() + "r");
        right.setBranchId(right.getParent().getBranchId(), BranchId.RIGHT);
        //        System.out.println(right.getBranchNumber()+" primeiro da direita do
        // nvel " + right.getLevel()
        //                + " :" + n);
        //        System.out.println(right.getBranchNumber()+ n);
        showNode(right, n);

        return right;

    }

    after(ProofTree pt, Node n): 
        ProofTreeAddNode(pt, n){
//        System.out.println(thisJoinPoint.getSignature());
        //        System.out.println(pt.getBranchNumber()+" mais um n do nvel " +
        // pt.getLevel() + " :" + n);
        //        System.out.println(pt.getBranchNumber()+ n);
        showNode(pt, n);

    }

    void showNode(ProofTree pt, Node n) {
//        System.out.println("<level: " +pt.getLevel() + " >"); 
//        System.out.println("<branch: " +pt.getBranchId() + " >"); 
//        System.out.println("<nodeNumber:  " + (pt.getNumberOfLocalNodes()-1) + " >"); 
//        System.out.println("<branchDescription: " +
//                (pt.getBranchNumber()==""?"rt":pt.getBranchNumber()) + " >"); 
//        System.out.println(n); 
//        System.out.println(pt.getLevel() + " " + pt.getNumberOfLocalNodes()+  
//                " " + pt.getBranchId() + " " + pt.getBranchNumber() + " " + n);
//
////        try {
////            System.in.read();
////        } catch (IOException e) {
////        }
//
    } 

}