/*
 * Created on 07/04/2005
 *
 */
package main.aspects;

import java.io.FileWriter;
import java.text.NumberFormat;

import logic.formulas.CompositeFormula;
import logic.formulas.Formula;
import logic.problem.Problem;
import logic.problem.ProblemType;
import logic.signedFormulas.SignedFormula;
import logicalSystems.classicalLogic.ClassicalConnectives;
import logicalSystems.classicalLogic.ClassicalSigns;
import main.facade.TableauFacade;
import main.proofTree.Node;
import main.proofTree.ProofTree;
import main.proofTree.SignedFormulaNode;
import main.strategy.memorySaver.MemorySaverStrategy;
import main.strategy.memorySaver.OptimizedClassicalProofTree;
import main.tableau.Proof;

/**
 * ONLY FOR MEMORY SAVER STRATEGY
 * 
 * @author Adolfo Gustavo Serra Seca Neto
 * 
 * 
 */
public aspect ProofTreeOnTime {
    declare precedence: ProofTreeHeightAspect, ProofTreeOnTime, TableauFacadeProfilerAspect;

    /** IF MEMORY SAVER STRATEGY */

    /** MEDIR TAMANHO DE OUTRO MODO */

    /** INCLUI PRIMEIRA FRMULA DO PROBLEMA 3 VEZES!!! */

    /**
     * EXCLUI T_TOP E F_BOTTOM mas no exclui repetidos! Ver profiler para fazer
     * igual.
     */

    TableauFacade currentTF;

    Problem currentProblem;

    before(TableauFacade tf, Problem p): 
        TableauFacadeProfilerAspect.createProof(tf, p){
        currentTF = tf;
        currentProblem = p;
    }

    int i = 0, provaIndex = 0;

    FileWriter fw = null;

    private MemorySaverStrategy _mockStrategy = new MemorySaverStrategy(null);

    private String _memorySaverStrategyName = _mockStrategy.getName()
            .replaceFirst(
                    _mockStrategy.getClass().getPackage().getName() + ".", "");

    private boolean isUsingMemorySaverStrategy() {
        return currentTF.getStrategyName().equals(_memorySaverStrategyName);
    }

    /**
     * complete proofs cannot be saved using MemorySaverStrategy because the
     * full proof tree is not kept in memory
     */
    void around():
        call(private void TableauFacadeProfilerAspect.saveProof(..) 	)   {
        if (isUsingMemorySaverStrategy() && currentTF.isSaveProof()) {
            System.err
                    .println("It is not possible to save complete proofs using "
                            + _memorySaverStrategyName);
        } else {
            proceed();
        }
    }

    /**
     * simplified proofs are saved step-by-step when using MemorySaverStrategy
     * also because the full proof tree is not kept in memory
     */
    void around(TableauFacade tf): args(tf,*) && 
        call(private void TableauFacadeProfilerAspect.saveSimplifiedProof(..) 	)   {
        if (!isUsingMemorySaverStrategy() && tf.isSaveSimpleProof()) {
            proceed(tf);
        }
    }

    /** Summaries are restricted when using MemorySaverStrategy */
    String around(TableauFacade tf, long timeSpent, NumberFormat nf, Proof p,
            Problem problem): args(tf, timeSpent, nf, p, problem) &&
           execution(private String TableauFacadeProfilerAspect.generateProofSummary(..) 	)   {
        if (!isUsingMemorySaverStrategy()) {
            return proceed(tf, timeSpent, nf, p, problem);
        }
        // if ()
        // . simplified proof tree length in bytes
        // . time spent
        // . number of nodes
        // . height
        // . formula factory complexity
        // . signed formula factory complexity
        // . size of problem
        // . size of proof tree
        String result = TableauFacadeProfilerAspect._currentProblemSummaryString
                + tf.getTexSeparator();
        result += "n/a" + tf.getTexSeparator();
        result += nf.format((timeSpent) / 1000.0) + tf.getTexSeparator();
        result += "n/a" + tf.getTexSeparator();
        result += "n/a" + tf.getTexSeparator();
        result += problem.getFormulaFactory().getComplexity()
                + tf.getTexSeparator();
        result += problem.getSignedFormulaFactory().getComplexity()
                + tf.getTexSeparator();
        result += TableauFacadeProfilerAspect.getSize(problem.getFormulas())
                + tf.getTexSeparator();
        result += "n/a";

        return result + tf.getTexFinalizer() + TableauFacade.LS;
    }

    /** Gets info about each problem in a sequence - BEGIN */
    /** Most taken from TableauFacadeProfilerAspect */
    private static final int NO_SIZE = -1;

    public static String _currentProblemName, _currentProblemInstanceName,
            _currentProblemFileName, _currentProblemSummaryString;

    int _currentProblemSize, _currentProblemBase;

    ProblemType _currentProblemType;

    // TODO instantiate a class
    /**
     * records info about problem in aspect attributes - for families of
     * problems
     */
    before(TableauFacade tf, String familyName, int size, int base,
            ProblemType type):  
                TableauFacadeProfilerAspect.problemInfo(tf, familyName, size, base, type){
        _currentProblemName = familyName;
        _currentProblemInstanceName = tf.generateFullProblemName(familyName,
                size, base, type);
        _currentProblemFileName = tf.generateFileName(familyName, size, base,
                type);
        _currentProblemSummaryString = familyName + tf.getTexSeparator() + size;
        _currentProblemSize = size;
        _currentProblemBase = base;
        _currentProblemType = type;
    }

    /** records info about problem in aspect attributes - for isolated problems */
    before(TableauFacade tf, String problemName):  
        TableauFacadeProfilerAspect.problemInfoWithoutSize(tf, problemName){
        _currentProblemName = problemName;
        _currentProblemInstanceName = tf.generateFullProblemName(problemName);
        _currentProblemFileName = tf.generateFileName(problemName);
        _currentProblemSummaryString = problemName;
        _currentProblemSize = NO_SIZE;
    }

    /** Gets info about each problem in a sequence - END */

    before(TableauFacade tf): this (tf) && call(Proof.new(..)) && !(cflowbelow(call(Proof.new(..)))) {
        if (isUsingMemorySaverStrategy() && tf.isSaveSimpleProof()) {
            String completeFileName;
            String specialTermination = ".proof.txt";

            /**
             * Verifies if it is an isolated problem or a sequence problem.
             * Should be an aspect by itself.
             */
            if (_currentProblemSize == NO_SIZE) {
                completeFileName = currentTF.getProblemsOutputDir()
                        + currentProblem.getName() + specialTermination;
            } else {
                completeFileName = currentTF.getProblemsOutputDir()
                        + currentProblem.getName()
                        + TableauFacade.FS
                        + currentTF.generateFullProblemName(currentProblem
                                .getName(), _currentProblemSize,
                                _currentProblemBase, _currentProblemType)
                        + specialTermination;
            }

            try {
                fw = new FileWriter(completeFileName);
                // writeProblemIn(fw);
            } catch (Exception e) {
            }
        }
    }

    private void writeProblemIn(FileWriter fw) throws Exception {
        for (int i = 0; i < currentProblem.getFormulas().size(); i++) {
            fw.write("p: " + currentProblem.getFormulas().get(i)
                    + System.getProperty("line.separator"));
        }
    }

    after(TableauFacade tf): this(tf) && call(Proof.new(..)) {
        if (isUsingMemorySaverStrategy() && tf.isSaveSimpleProof()) {
            try {
                fw.close();
            } catch (Exception e) {
            }
        }
    }

    // before(ProofTree pt, SignedFormulaNode n): target(pt) &&
    // call(ProofTree+.new(..)) && args(n){
    //        
    // System.out.println(" FFFF "+n);
    // if (isUsingMemorySaverStrategy()) {
    // System.out.println(" FFFF "+n);
    // }
    // }

    before(ProofTree pt, Node n): target(OptimizedClassicalProofTree+) && target(pt) && 
    call(* ProofTree+.addLast(..)) && args(n){
        if (isUsingMemorySaverStrategy() && currentTF.isSaveSimpleProof()) {
            try {
                /** root node */
                SignedFormula sf = ((SignedFormula) ((SignedFormulaNode) pt
                        .getRoot()).getContent());
                Formula f = sf.getFormula();

                boolean isT_TOP, isF_BOTTOM;
                // boolean isT_TOP = ((sf.getSign() == ClassicalSigns.TRUE)
                // && (f instanceof CompositeFormula) && ((CompositeFormula) f)
                // .getConnective() == ClassicalConnectives.TOP);
                // boolean isF_BOTTOM = ((sf.getSign() == ClassicalSigns.FALSE)
                // && (f instanceof CompositeFormula) && ((CompositeFormula) f)
                // .getConnective() == ClassicalConnectives.BOTTOM);

                /**
                 * if the node to which this node is being added is the root
                 * node of its proof tree, and it is not T_TOP, F_BOTTOM or
                 */
                if ((pt.getRoot() == pt.getCurrent())
                // && !isT_TOP
                // && !isF_BOTTOM
                ) {

                    fw.write(pt.getBranchNumber()
                            + " "
                            + ((SignedFormulaNode) pt.getRoot()).getContent()
                                    .toString()
                            + System.getProperty("line.separator"));
                }

                /** Node being added */
                sf = ((SignedFormula) ((SignedFormulaNode) n).getContent());
                f = sf.getFormula();

                isT_TOP = ((sf.getSign() == ClassicalSigns.TRUE)
                        && (f instanceof CompositeFormula) && ((CompositeFormula) f)
                        .getConnective() == ClassicalConnectives.TOP);
                isF_BOTTOM = ((sf.getSign() == ClassicalSigns.FALSE)
                        && (f instanceof CompositeFormula) && ((CompositeFormula) f)
                        .getConnective() == ClassicalConnectives.BOTTOM);

                // if (!isT_TOP && !isF_BOTTOM) {
                fw.write(pt.getBranchNumber() + " "
                        + ((SignedFormulaNode) n).getContent().toString()
                        + System.getProperty("line.separator"));
                fw.flush();
                // }
            } catch (Exception e) {
            }
        }
    }

    // after(ProofTree pt, Node n): target(pt) && call(* ProofTree+.addLeft(..))
    // && args(n){
    // if (isUsingMemorySaverStrategy()) {
    // try {
    // fw.write(pt.getBranchNumber() + "l "
    // + ((SignedFormulaNode) n).getContent().toString()
    // + System.getProperty("line.separator"));
    // fw.flush();
    // } catch (Exception e) {
    // }
    // }
    // }

    // after(ProofTree pt, Node n): target(pt) && call(*
    // ProofTree+.addRight(..)) && args(n){
    // if (isUsingMemorySaverStrategy()) {
    // try {
    // fw.write(pt.getBranchNumber() + " PB r "
    // + ((SignedFormulaNode) n).getContent().toString()
    // + System.getProperty("line.separator"));
    // fw.flush();
    // } catch (Exception e) {
    // }
    // }
    // }

    // before(ClassicalProofTree pt): target(pt) &&
    // call (* ClassicalProofTree+.eraseLeft()){
    //
    // // if (pt.getLeft() != null) {
    // try {
    // fw.write("XXX " + pt.getLeft().getBranchNumber() + " "
    // + ((ClassicalProofTree) pt.getLeft()).isClosed()
    // + System.getProperty("line.separator"));
    // fw.flush();
    // } catch (Exception e) {
    //
    // }
    // // }
    // // System.err.println("VAI APAGAR: "
    // // + ((ClassicalProofTree) pt.getLeft()).isClosed()
    // // + ((ClassicalProofTree) pt.getLeft()).getBranchNumber());
    // }

    boolean around(OptimizedClassicalProofTree pt): target (pt) &&
    	(call (boolean  OptimizedClassicalProofTree+.isClosedByBranching())) &&
          !(within (ProofTreeOnTime))
    {

        if (isUsingMemorySaverStrategy() && currentTF.isSaveSimpleProof()) {

            boolean closedBefore = pt.getClosedByBranching();

            boolean result = proceed(pt);

            if (!closedBefore && result) {
                // System.err.println("XXX " + pt.getBranchNumber());
                try {
                    fw.write(pt.getBranchNumber() + " "
                    // + pt.isClosed()
                            + "children closed"
                            + System.getProperty("line.separator"));
                    fw.flush();
                } catch (Exception e) {

                }
            }

            return result;
        } else
            return proceed(pt);

    }

    // boolean around(ClassicalProofTree pt): target (pt) && (call (boolean
    // ClassicalProofTree+.isClosed()) &&
    // // (
    // (withincode (* Strategy+.close(..)))
    // // ||
    // // (
    // // (cflowbelow (execution (* ClassicalProofTree+.isClosed(..))))
    // // &&
    // // !(cflow (execution (* ClassicalProofTree+.isClosedByBranching(..))))
    // // &&
    // // !(within (ProofTreeOnTime))
    // // )
    // // )
    // ){
    //
    // // if (pt.getLeft()==NullClosedProofTree.INSTANCE){
    // // System.out.println(pt.getBranchNumber());
    // // }
    //
    // // if (pt.getBranchNumber().equals("rr")) {
    // // System.err.println(pt.getLeft() == null);
    // // System.err.println(pt.getRight() == null);
    // // //
    // System.err.println((((ClassicalProofTree)pt.getLeft()).isClosed()));
    // // //
    // System.err.println((((ClassicalProofTree)pt.getRight()).isClosed()));
    // // }
    //
    // if (!pt.isLocallyClosed() && (pt.getLeft() != null)
    // && (((ClassicalProofTree) pt.getLeft()).isClosed())
    // && (pt.getRight() != null)
    // && (((ClassicalProofTree) pt.getRight()).isClosed())) {
    // try {
    // fw.write(pt.getBranchNumber() + " "
    // // + pt.isClosed()
    // + "children closed"
    // + System.getProperty("line.separator"));
    // fw.flush();
    // } catch (Exception e) {
    //
    // }
    //
    // }
    // return proceed(pt);
    //
    // // {
    // // try {
    // // fw.write(pt.getBranchNumber() + " " + pt.isClosed()
    // // + System.getProperty("line.separator"));
    // // fw.flush();
    // // } catch (Exception e) {
    // //
    // // }
    // // return proceed(pt);
    // }

    // boolean around(ClassicalProofTree pt): target (pt) && (call (boolean
    // ClassicalProofTree+.isClosed()) && (within (strategy.*)))
    // {
    // boolean closedByBranchingBefore = pt.getClosedByBranching();
    //
    // // if (pt.getBranchNumber().equals("llll")) {
    // // System.out.println(" llll " + closedByBranchingBefore);
    // // }
    //
    // boolean result = proceed(pt);
    // boolean closedByBranchingAfter = pt.isClosedByBranching();
    //
    // // if (pt.getBranchNumber().equals("llll")) {
    // // System.out.println(" llll " + closedByBranchingAfter);
    // // }
    //
    // if (!closedByBranchingBefore && closedByBranchingAfter) {
    // // System.err.println(thisJoinPoint.getThis());
    // // System.out.println("HERE " + pt.getBranchNumber() + " " +
    // // result);
    // try {
    // fw.write(pt.getBranchNumber() + " " + pt.isClosed()
    // + System.getProperty("line.separator"));
    // fw.flush();
    // } catch (Exception e) {
    //
    // }
    // }
    // return result;
    // }

    pointcut setClosed(OptimizedClassicalProofTree pt): this(pt) && 
    	set (* OptimizedClassicalProofTree+._locallyClosed);

    // void around(ClassicalProofTree pt): setClosed(pt){
    // boolean beforeSet = pt.isClosed();
    // proceed(pt);
    //
    // if (!beforeSet && pt.isClosed()) {
    // // System.out.println(pt.getBranchNumber() + " " + pt.isClosed());
    // try{
    // fw.write(pt.getBranchNumber() + " " + pt.isClosed()
    // + System.getProperty("line.separator"));
    // fw.flush();
    // }catch(Exception e){
    //                
    // }
    // }
    // }

    void around(OptimizedClassicalProofTree pt, Node n): target(pt) && args(n) &&
    				execution(* OptimizedClassicalProofTree+.updateMultimap(..)) {
        if (isUsingMemorySaverStrategy() && currentTF.isSaveSimpleProof()) {
            boolean beforeUpdate = pt.isClosed();
//            System.err.println("OI 1 " + n + " " + pt.getBranchNumber() + " " + pt.isClosed());
            proceed(pt, n);
//            System.err.println(pt.getFsmm());
//            System.err.println("OI 2 " + n + " " + pt.getBranchNumber() + " " +pt.isClosed());

            if (!beforeUpdate && pt.isClosed()) {
                try {
                    fw.write(pt.getBranchNumber() + " " +
                    // pt.isClosed() +
                            " closed by "
                            + ((SignedFormula) n.getContent()).getFormula()
                            + System.getProperty("line.separator"));
                    fw.flush();
                } catch (Exception e) {

                }
            }
        } else
            proceed(pt, n);
    }
}
