/*
 * Created on 01/08/2005
 *
 */
package main.facade;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import exceptions.TPException;

/**
 * Concentrator of configurations.
 * 
 * @author Adolfo Gustavo Serra Seca Neto
 * 
 *  
 */
public class ConfigurationConcentrator {

	private static ConfigurationConcentrator __concentrador;

	private Properties properties;

	private String propertiesFileName = "application.properties";

	/**
	 * Creates a configuration concentrator.
	 */
	private ConfigurationConcentrator() {
		properties = new Properties();

		try {
			InputStream propertiesFile = new FileInputStream(propertiesFileName);
			properties.load(propertiesFile);
		} catch (FileNotFoundException e) {
			throw new TPException("Properties file (" + propertiesFileName
					+ ") not found.");
		} catch (IOException e) {
			throw new TPException("Problem reading properties file ("
					+ propertiesFileName + ").");
		}
	}

	/**
	 * gets the unique instance of this class
	 * 
	 * @return a configuration concentrator
	 */
	public static ConfigurationConcentrator getInstance() {
		if (__concentrador == null) {
			__concentrador = new ConfigurationConcentrator();
		}

		return __concentrador;
	}

	/**
	 * gets a property
	 * 
	 * @param propertyName
	 * @return property value string
	 *  
	 */
	public String getProperty(String propertyName) {
		String propertyValue = properties.getProperty(propertyName);

		if (propertyValue == null) {
			throw new TPException("Property " + propertyName
					+ " not defined in " + propertiesFileName);
		}
		return propertyValue;
	}
}
