/*
 * Created on 15/03/2005
 *
 */
package main.facade;

import java.io.FileInputStream;
import java.util.Properties;

/**
 * Uses the following directory structure based on a homeDir to get complete
 * paths for files: <BR>
 * lib <BR>
 * lib/ext - external libraies <BR>
 * lib/generated - libraries generated by the project <BR>
 * problems <BR>
 * problems/collected - problems copied from somewhere <BR>
 * problems/created - created by myself <BR>
 * problems/downloaded - downlpaded <BR>
 * problems/generated - generated by problemGenerator <BR>
 * results <BR>
 * results/proofs - proof files <BR>
 * results/summaries - summaries of proofs and analysis <BR>
 * config - configuration files <BR>
 * 
 * @author Adolfo Gustavo Serra Seca Neto
 *  
 */
public class DirectoryStructure {

    public static String SYSTEM_NAME = "TableauProver";

    public static final String FS = System.getProperty("file.separator");

    public static String LIB = "lib";

    public static String EXT = "ext";

    public static String GENERATED = "generated";

    public static String PROBLEMS = "problems";

    public static String COLLECTED = "collected";

    public static String CREATED = "created";

    public static String DOWNLOADED = "downloaded";

    public static String RESULTS = "results";

    public static String PROOFS = "proofs";

    public static String SUMMARIES = "summaries";

    public static String CONFIG = "config";

    public static String PROPERTIES_FILE = "DS_properties.txt";

    private String homeDir;

    private String defaultHomeDir = System.getProperty("user.home")
            + DirectoryStructure.FS + DirectoryStructure.SYSTEM_NAME;

    /**
     * creates a DirectoryStructure based on a homeDir
     * 
     * @param homeDir
     */
    public DirectoryStructure(String homeDir) {
        this.homeDir = homeDir;
    }

    /**
     * creates a DirectoryStructure based on a homeDir in a properties file (DS_properties.txt) or
     * on a default homedir (user.home/SYSTEM_NAME)
     */
    public DirectoryStructure() {
        try {
            this.homeDir = getDefaultProperties().getProperty("homeDir");
        } catch (Exception e) {
            this.homeDir = defaultHomeDir;
        }
    }

    private Properties getDefaultProperties() throws Exception {

        Properties prop = new Properties();

        String completeFilename = System.getProperty("user.home")
                + DirectoryStructure.FS + DirectoryStructure.SYSTEM_NAME
                + DirectoryStructure.FS + DirectoryStructure.CONFIG
                + DirectoryStructure.FS + DirectoryStructure.PROPERTIES_FILE;

        FileInputStream fis = new FileInputStream(completeFilename);

        prop.load(fis);

        return prop;
    }
    
    /**
     * @return home directory
     */
    public String getHomeDir(){
        return homeDir;
    }

    /**
     * @param subPath
     * @return complete path for subpath
     */
    public String getCompletePath(String subPath) {
        return homeDir + FS + subPath + FS;
    }

    /**
     * @param subPath1
     * @param subPath2
     * @return complete path for subpaths
     */
    public String getCompletePath(String subPath1, String subPath2) {
        return homeDir + FS + subPath1 + FS + subPath2 + FS;
    }

    /**
     * @param subPath1
     * @param subPath2
     * @param subPath3
     * @return complete path for subpaths
     */
    public String getCompletePath(String subPath1, String subPath2,
            String subPath3) {
        return homeDir + FS + subPath1 + FS + subPath2 + FS + subPath3 + FS;
    }

    /**
     * @param subPath1
     * @param subPath2
     * @param subPath3
     * @param subPath4
     * @return complete path for subpaths
     */
    public String getCompletePath(String subPath1, String subPath2,
            String subPath3, String subPath4) {
        return homeDir + FS + subPath1 + FS + subPath2 + FS + subPath3 + FS
                + subPath4 + FS;
    }
}