/*
 * Created on 07/03/2005
 *
 */
package main.facade;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;

import main.proofTree.Node;
import main.proofTree.ProofTree;
import main.proofTree.StringNode;
import main.proofTree.iterator.ProofTreeLocalIterator;

/**
 * 
 * @author Adolfo Gustavo Serra Seca neto
 *  
 */
public class ProofJTree extends JPanel implements ActionListener {

    private static String ADD_COMMAND = "add";

    private JTextField newNodeText;

    private ProofTree proofTree;

    private DynamicTree visualTree;

    private static StringNode ROOT = new StringNode("root");

    private static StringNode LEFT = new StringNode("left");

    private static StringNode RIGHT = new StringNode("right");

    public ProofJTree(ProofTree proofTree) {
        super(new BorderLayout());

        this.proofTree = proofTree;

        //Create the components.
        visualTree = new DynamicTree(ROOT);
        populateVisualTree(visualTree, null, proofTree);
        visualTree.expandAll(true);

        //Lay everything out.
        visualTree.setPreferredSize(new Dimension(600, 300));
        add(visualTree, BorderLayout.CENTER);

        JButton addButton = new JButton("Add a node");
        addButton.setActionCommand(ADD_COMMAND);
        addButton.addActionListener(this);

        newNodeText = new JTextField("New node label");

        JPanel panel = new JPanel(new GridLayout(0, 1));
        panel.add(addButton);
        panel.add(newNodeText);
        add(panel, BorderLayout.LINE_END);

    }

    private void populateVisualTree(DynamicTree treePanel,
            DefaultMutableTreeNode current, ProofTree tree) {

        // gets elements from current branch
        ProofTreeLocalIterator treeIterator = tree.getLocalIterator();

        while (treeIterator.hasNext()) {
            treePanel.addObject(current, treeIterator.next());
        }

        // gets elements from other branches, if any
        if (tree.getLeft() != null) {
            DefaultMutableTreeNode left = treePanel.addObject(current, LEFT);
            populateVisualTree(treePanel, left, tree.getLeft());
        }
        if (tree.getRight() != null) {
            DefaultMutableTreeNode right = treePanel.addObject(current, RIGHT);
            populateVisualTree(treePanel, right, tree.getRight());
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();

        if (ADD_COMMAND.equals(command)) {
            insert(new StringNode(newNodeText.getText()));
        }

    }

    /**
     * @param node
     */
    private void insert(Object obj) {

        // update visual tree
        DefaultMutableTreeNode parent = (DefaultMutableTreeNode) visualTree
                .getCurrentNode();

        if ((parent.getUserObject() == LEFT)
                || (parent.getUserObject() == RIGHT)
                || (parent.getUserObject() == ROOT)) {

            visualTree.addObject(parent, obj, calculatePosition(parent));

            // updates proof tree
            if (parent.getUserObject() == ROOT) {
                proofTree.getRoot().getBranch().addLast((Node) obj);
            } else {
                if (parent.getUserObject() == LEFT) {
                    DefaultMutableTreeNode left = (DefaultMutableTreeNode) (visualTree
                            .locateObjectNode(obj));

                    DefaultMutableTreeNode brother = (DefaultMutableTreeNode) ((DefaultMutableTreeNode) parent
                            .getParent()).getChildAt(0);

                    ((Node) brother.getUserObject()).getBranch().getLeft()
                            .addLast((Node) obj);
                } else {
                    DefaultMutableTreeNode left = (DefaultMutableTreeNode) (visualTree
                            .locateObjectNode(obj));

                    DefaultMutableTreeNode brother = (DefaultMutableTreeNode) ((DefaultMutableTreeNode) parent
                            .getParent()).getChildAt(0);

                    ((Node) brother.getUserObject()).getBranch().getRight()
                            .addLast((Node) obj);
                }
            }
        } else {
            // if parent has no left/right children

            if (((DefaultMutableTreeNode) parent.getParent()).getDepth() == 1) {

                // update visual tree
                // insert two branches in the current branch
                DefaultMutableTreeNode left = visualTree
                        .addObject((DefaultMutableTreeNode) parent.getParent(),
                                LEFT, true);
                visualTree.addObject(left, obj, true);
                DefaultMutableTreeNode right = visualTree.addObject(
                        (DefaultMutableTreeNode) parent.getParent(), RIGHT,
                        true);
                visualTree.addObject(right, obj, true);

                // update proof tree
                Node nodeInTheBranch = (Node) parent.getUserObject();
                nodeInTheBranch.getBranch().addLeft((Node) obj);
                nodeInTheBranch.getBranch().addRight((Node) obj);
//                System.out.println(proofTree);

            }
        }
    }

    public int calculatePosition(DefaultMutableTreeNode node) {
        int i = 0;
        for (Enumeration e = node.children(); e.hasMoreElements(); i++) {
            TreeNode n = (TreeNode) e.nextElement();
            if (!n.isLeaf()) {
                return i;
            }
        }
        return i;
    }

    public static void main(String args[]) {
        //Schedule a job for the event-dispatching thread:
        //creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });

    }

    /**
     * Create the GUI and show it. For thread safety, this method should be
     * invoked from the event-dispatching thread.
     */
    private static void createAndShowGUI() {

        StringNode aNode1, aNode2;
        ProofTree pt, left, right, leftleft, leftright, rightright, rightleft;

        pt = new ProofTree(new StringNode("root 0"));

        pt.addLast(new StringNode("root 1"));
        pt.addLast(new StringNode("root 2"));
        left = pt.addLeft(new StringNode("left 0"));
        left.addLast(new StringNode("left 1"));
        leftleft = left.addLeft(aNode2 = new StringNode("left left 0"));
        leftright = left.addRight(new StringNode("left right 0"));
        right = pt.addRight(new StringNode("right 0"));
        right.addLast(new StringNode("right 1"));
        rightright = right.addRight(new StringNode("right right 0"));
        rightleft = right.addLeft(aNode1 = new StringNode("right left 0"));

        ProofJTree pjt = new ProofJTree(pt);

        //Make sure we have nice window decorations.
        JFrame.setDefaultLookAndFeelDecorated(true);

        //Create and set up the window.
        JFrame frame = new JFrame("ProofJTree Example");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        JPanel jp = new JPanel(new BorderLayout());
        jp.add(pjt, BorderLayout.CENTER);
        //        pjt.getVisualTree().expandAll(true);

        jp.setOpaque(true);
        frame.setContentPane(jp);

        //Display the window.
        frame.pack();
        frame.setVisible(true);
    }

}