/*
 * Created on 24/09/2004
 *
 */
package main.proofTree.iterator;

import main.proofTree.Node;
import main.proofTree.ProofTree;

/**
 * 
 * Iterator for ProofTrees that allows one to see other branches. The traversal
 * can be bottom-up or top-down.
 * 
 * @author Adolfo Gustavo Serra Seca Neto
 *  
 */
public class ProofTreeGlobalIterator implements ProofTreeAdvancedIterator {

	ProofTree pt, currentPT;

	Node current;

	ProofTreeBasicIterator pt_it;

	public ProofTreeGlobalIterator(ProofTree pt) {
		this.pt = pt;
		this.currentPT = pt;
		this.current = currentPT.getRoot();
		this.pt_it = pt.getLocalIterator();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see proofTree.ProofTreeAdvancedIterator#hasPrevious()
	 */
	public boolean hasPrevious() {
		return pt_it.hasPrevious() || currentPT.getParent() != null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see proofTree.ProofTreeAdvancedIterator#previous()
	 */
	public Node previous() {
		if (pt_it.hasPrevious()) {
			return current = pt_it.previous();
		} else if (currentPT.getParent() != null) {
			currentPT = currentPT.getParent();
			pt_it = currentPT.getBackwardLocalIterator();
			return current = pt_it.previous();
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see proofTree.ProofTreeAdvancedIterator#current()
	 */
	public Node current() {
		return current;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see proofTree.ProofTreeAdvancedIterator#hasNextLeft()
	 */
	public boolean hasNextLeft() {
//	    System.err.println("PTAI"+(pt_it.hasNext()));
//	    System.err.println("PTAI"+(currentPT.getLeft()!=null));
		return pt_it.hasNext() || currentPT.getLeft() != null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see proofTree.ProofTreeAdvancedIterator#hasNextRight()
	 */
	public boolean hasNextRight() {
		return pt_it.hasNext() || currentPT.getRight() != null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see proofTree.ProofTreeAdvancedIterator#nextLeft()
	 */
	public Node nextLeft() {
		if (pt_it.hasNext()) {
			return current = pt_it.next();
		} else if (currentPT.getLeft() != null) {
			currentPT = currentPT.getLeft();
			pt_it = currentPT.getLocalIterator();
			return current = pt_it.next();
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see proofTree.ProofTreeAdvancedIterator#nextRight()
	 */
	public Node nextRight() {
		if (pt_it.hasNext()) {
			return current = pt_it.next();
		} else if (currentPT.getRight() != null) {
			currentPT = currentPT.getRight();
			pt_it = currentPT.getLocalIterator();
			return current = pt_it.next();
		}
		return null;
	}

}