/*
 * Created on 29/09/2005
 *
 */
package main.strategy;

import java.util.LinkedList;

import logic.signedFormulas.SignedFormulaBuilder;
import main.proofTree.Origin;
import main.proofTree.ProofTree;
import main.proofTree.SignedFormulaNode;
import main.tableau.Method;
import rules.Rule;

/**
 * Features of several strategies.
 * 
 * @author Adolfo Gustavo Serra Seca Neto
 *  
 */
public abstract class AbstractStrategy extends Strategy {

	/**
	 * Creates an AbstractSimpleStrategy
	 * 
	 * @param m
	 */
	public AbstractStrategy(Method m) {
		super(m);
	}

	/**
	 * current proof tree
	 */
	private ClassicalProofTree _current;

	/**
	 * @return current proof tree
	 */
	public ClassicalProofTree getCurrent() {
		return _current;
	}

	/**
	 * sets current proof tree
	 * 
	 * @param value
	 */
	public void setCurrent(ClassicalProofTree value) {
		_current = value;
	}

	/**
	 * returns main proof tree
	 */
	private ClassicalProofTree _proofTree;

	/**
	 * @return main proof tree
	 */
	public ClassicalProofTree getProofTree() {
		return _proofTree;
	}

	/**
	 * sets main proof tree
	 * 
	 * @param value
	 */
	public void setProofTree(ClassicalProofTree value) {
		_proofTree = value;
	}

	/**
	 * Crates an origin object
	 * 
	 * @param rule
	 * @param main
	 * @param auxiliary
	 * @return the origin object
	 */
	protected abstract Origin createOrigin(Rule rule, SignedFormulaNode main,
			SignedFormulaNode auxiliary);

	/**
	 * @return true if it is the right branch of a node whose left branch is
	 *         open
	 */
	protected abstract boolean isRightBranchOfOpenLeftBranch();

	/**
	 * @param openBranches
	 * @param sfb
	 * @return
	 */
	abstract protected boolean applyPBRule(LinkedList openBranches,
			SignedFormulaBuilder sfb);


	/**
	 * @param thisProofTree
	 */
	protected final void initialPBCandidatesActions(
			ClassicalProofTree thisProofTree) {
		thisProofTree.removeFromPBCandidates(0);
		thisProofTree.removeFromPBCandidates(0);
	}

	/**
	 * possibly erases left branch of the current proof tree
	 */
	abstract protected void maybeEraseLeft();
	
    protected final ProofTree execute(ProofTree cpt, SignedFormulaBuilder sfb) {
    	ClassicalProofTree pt = (ClassicalProofTree) cpt;
        
        setProofTree(pt);

        // the close method has already loaded the problem into a proof tree!
        // if it is already closed, finish now!
        if (getProofTree().isClosed())
            return pt;

        initialPBCandidatesActions(getProofTree());

        LinkedList openBranches = new LinkedList();
        openBranches.addFirst(pt);

        boolean hasApplied;
        while (!openBranches.isEmpty() && !getProofTree().isClosed()) {

            setCurrent((ClassicalProofTree)openBranches
                    .removeFirst());

            // if this branch is the right of some branch
            if (isRightBranchOfOpenLeftBranch()) {
                break;
                //                System.err.println("Did not close because left " +_current
                //                        .getParent().getLeft().getBranchNumber() + " is open");
            }
            
            maybeEraseLeft();

            // keep applying rules until it is not possible to apply
            do {
                hasApplied = applyLinearRules(sfb);
                if (getCurrent().isClosed()) {
                    //                    System.err.println("Closed " +_current
                    //                            .getBranchNumber() + " !");
                    break;
                } else {
                    hasApplied = applyPBRule(openBranches, sfb);
                }

            } while (hasApplied);


        }

        return getProofTree();
    }

	/**
	 * @param sfb
	 * @return
	 */
	protected abstract boolean applyLinearRules(SignedFormulaBuilder sfb);


}
