/*
 * Created on 19/11/2004
 *
 */
package main.strategy;

import java.util.HashMap;
import java.util.Map;

import logic.formulas.CompositeFormula;
import logic.signedFormulas.SignedFormula;
import logic.signedFormulas.SignedFormulaList;
import logic.structures.FormulaSignMultimap;
import main.proofTree.Node;
import main.proofTree.ProofTree;
import main.proofTree.SignedFormulaNode;
import main.proofTree.SignedFormulaNodeState;

/** Class that represents a proof tree than can be closed.
 * @author adolfo
 * 
 */
public abstract class ClassicalProofTree extends ProofTree  implements CloseableProofTree{

    FormulaSignMultimap _fsmm;

    SignedFormulaList _PBCandidates;

    Map _signedFormulasToNodes;

    protected ClassicalProofTree() {
    };

    /** Creates a ClassicalProofTree with an initial node
     * @param aNode
     */
    public ClassicalProofTree(SignedFormulaNode aNode) {
        super(aNode);

        _fsmm = new FormulaSignMultimap();
        _PBCandidates = new SignedFormulaList();
        _signedFormulasToNodes = new HashMap();

        updateStructures(aNode);
    }

    
    
    /*
     * (non-Javadoc)
     * 
     * @see proofTree.ProofTree#addLast(proofTree.Node)
     */
    public void addLast(Node aNode) {
        // only adds if it is new
        if (getNode((SignedFormula) aNode.getContent()) == null) {
            super.addLast(aNode);
            updateStructures(aNode);
        }
    }

    // TODO
    // addLeft and addRight should only add a new node when it is new? 
    // This is guaranteed by PB features?
    
    /* (non-Javadoc)
     * @see proofTree.ProofTree#setOtherStructures(proofTree.Node)
     */
    protected void setOtherStructures(ProofTree pt, Node aNode) {
        ((ClassicalProofTree) pt).addPBCandidates(this.getPBCandidates());
    }
    
//    /* (non-Javadoc)
//     * @see proofTree.ProofTree#makeInstance(proofTree.Node)
//     */
//    protected ProofTree makeInstance(Node aNode) {
//        return new ClassicalProofTree((SignedFormulaNode) aNode);
//    }

    public abstract boolean isClosed();
    
//    {
//        return isLocallyClosed() || isClosedByBranching();
//    }

    /**
     * @param aNode
     */
    protected void updateStructures(Node aNode) {
        updateMultimap((SignedFormulaNode) aNode);
        updatePBCandidates((SignedFormulaNode) aNode);
        updateSignedFormulasToNodesMap((SignedFormulaNode) aNode);
    }


    /**
     * @param node
     */
    protected  abstract void updateMultimap(SignedFormulaNode node);

    protected void updatePBCandidates(SignedFormulaNode aNode) {
        if (((SignedFormula) aNode.getContent()).getFormula() instanceof CompositeFormula) {
            _PBCandidates.add((SignedFormula) aNode.getContent());
        } else {
            aNode.setState(SignedFormulaNodeState.FULFILLED);
        }

    }

    protected void updateSignedFormulasToNodesMap(SignedFormulaNode aNode) {
        _signedFormulasToNodes.put(aNode.getContent(), aNode);
    }

    /**
     * @param candidates
     */
    protected void addPBCandidates(SignedFormulaList candidates) {
        _PBCandidates.addAll(candidates);
    }

    public void setPBCandidates(SignedFormulaList sfl) {
        _PBCandidates = sfl;
    }

    public void removeFromPBCandidates(SignedFormula sf) {
        _PBCandidates.remove(sf);
    }

    public void removeFromPBCandidates(int index) {
        _PBCandidates.remove(index);
    }

    public SignedFormulaList getPBCandidates() {
        return _PBCandidates;
    }

    public SignedFormulaNode getNode(SignedFormula formula) {

        SignedFormulaNode sfn = (SignedFormulaNode) _signedFormulasToNodes
                .get(formula);
        if (sfn != null) {
            return sfn;
        } else {
            if (getParent() != null) {
                return ((ClassicalProofTree) getParent()).getNode(formula);
            }
        }

        return null;
    }

    public void removeFromPBCandidates(SignedFormula sf,
            SignedFormulaNodeState state) {
        SignedFormulaNode sfn = getNode(sf);
        sfn.setState(state);
        removeFromPBCandidates(sf);

        // System.err.println ("after remove: "+ _PBCandidates);
    }

    /**
     * 
     */
    protected String getStatus() {
        return "closed? : " + isClosed() + System.getProperty("line.separator");
    }

    /**
     * @return Returns the _signedFormulasToNodes.
     */
    public Map getSignedFormulasToNodes() {
        return _signedFormulasToNodes;
    }

    /**
     * @return Returns the _fsmm.
     */
    public FormulaSignMultimap getFsmm() {
        return _fsmm;
    }

    /**
     *  erases the left branch
     */
    public void eraseLeft() {
    	// TODO Remove comment
//        setLeft(NullClosedProofTree.INSTANCE);
    }


}