/*
 * Created on 27/01/2005
 *
 */
package main.strategy.memorySaver;

import logic.formulas.AtomicFormula;
import logic.formulas.CompositeFormula;
import logic.formulas.Formula;
import logic.formulas.FormulaList;
import logic.signedFormulas.SignedFormula;
import logic.signedFormulas.SignedFormulaList;
import main.proofTree.iterator.ProofTreeBasicIterator;
import main.strategy.ClassicalProofTree;

/**
 * @author Adolfo Gustavo Serra Seca Neto
 *  
 */
public class ReferenceFinder {

    /**
     * @param cpt
     * @param aFormula
     * @return
     */
    public static SignedFormulaList getLocalReferences(ClassicalProofTree cpt,
            Formula aFormula) {

        SignedFormulaList sfl = new SignedFormulaList();
        ProofTreeBasicIterator pti = cpt.getLocalIterator();

//        System.out.println("gLR " + aFormula);

        while (pti.hasNext()) {
            SignedFormula sf = (SignedFormula) pti.next().getContent();

            if (aFormula.isSubformula(sf.getFormula())) {
                sfl.add(sf);
            }
        }

//        System.out.println(sfl);

        return sfl;
    }

    //    public static boolean contains(SignedFormula sf, Formula f) {
    //
    //        return contains(sf.getFormula(), f);
    //    }
    //
    //    public static boolean contains(Formula f1, Formula f2) {
    //
    //        if (f1 == f2) {
    //            return true;
    //        } else if (f1 instanceof AtomicFormula) {
    //            return false;
    //        } else {
    //            CompositeFormula cf = (CompositeFormula) f1;
    //
    //            for (int i = 0; i < cf.getImmediateSubformulas().size(); i++) {
    //                if (contains((Formula) cf.getImmediateSubformulas().get(i), f2)) {
    //                    return true;
    //                }
    //            }
    //        }
    //
    //        return false;
    //    }

    /**
     * @param proofTree
     * @param bottomFormula
     * @param signedFormulaWithTOPorBOTTOM
     * @return
     */
    public static FormulaList getSubformulaLocalReferences(
            ClassicalProofTree proofTree, Formula bottomFormula,
            SignedFormula signedFormulaWithTOPorBOTTOM) {
        return subformulasOfWith(signedFormulaWithTOPorBOTTOM.getFormula(),
                bottomFormula, new FormulaList());
    }

    public static FormulaList subformulasOfWith(Formula f1, Formula f2, FormulaList fl) {
        if (f1 instanceof AtomicFormula) {
            return fl;
        } else {
            CompositeFormula cf = (CompositeFormula) f1;

            // search immediate subformulas first
            for (int i = 0; i < cf.getImmediateSubformulas().size(); i++) {
                if (cf.getImmediateSubformulas().get(i) == f2) {
                    fl.add(cf);
                    return fl;
                }
            }

            // searches deeper subformulas first
            for (int i = 0; i < cf.getImmediateSubformulas().size(); i++) {
                FormulaList result =subformulasOfWith((Formula)cf.getImmediateSubformulas().get(i),f2,fl); 
                if (result.size()>0) {
                    return result;
                }
            }
        }

        return fl;
    }

//    /**
//     * @param proofTree
//     * @param formula
//     * @return
//     */
//    public static SignedFormulaList getParentReferences(
//            ClassicalProofTree proofTree, Formula formula) {
//
//        SignedFormulaList sfl = new SignedFormulaList();
//
//        System.out.println("Adolfo");
//
//        return sfl;
//    }

    /**
     * @param proofTree
     * @param formula
     * @param mainCandidate
     * @return
     */
    public static FormulaList getSubformulaParentReferences(
            ClassicalProofTree proofTree, Formula formula,
            SignedFormula mainCandidate) {
        return subformulasOfWith(mainCandidate.getFormula(),
                formula, new FormulaList());
    }
    
    public static SignedFormula getSignedFormulaWith (SignedFormulaList sfl, Formula f1, Formula f2){
        for (int i=0; i< sfl.size(); i++){
            if (f1.isStrictSubformula(sfl.get(i).getFormula()) || f2.isSubformula(sfl.get(i).getFormula())){
                return sfl.get(i);
            }
        }
        
        return null;
    }

    public static SignedFormula getSignedFormulaWith (SignedFormulaList sfl, Formula f1, int begin){
        for (int i=begin; i< sfl.size(); i++){
            if (f1.isStrictSubformula(sfl.get(i).getFormula()) ){
                return sfl.get(i);
            }
        }
        
        return null;
    }

}