/*
 * Created on 19/11/2004
 *
 */
package main.strategy.oldsimple;

import java.util.HashMap;
import java.util.Map;

import logic.formulas.CompositeFormula;
import logic.signedFormulas.SignedFormula;
import logic.signedFormulas.SignedFormulaList;
import logic.structures.FormulaSignMultimap;
import main.proofTree.Node;
import main.proofTree.ProofTree;
import main.proofTree.SignedFormulaNode;
import main.proofTree.SignedFormulaNodeState;
import main.strategy.CloseableProofTree;
import main.strategy.NullClosedProofTree;

/**
 * @author adolfo
 *  
 */
public class OldClassicalProofTree extends ProofTree implements CloseableProofTree{
    
    public OldClassicalProofTree(){
        
    }


    boolean _closed;

    FormulaSignMultimap _fsmm;

    SignedFormulaList _PBCandidates;

    Map _signedFormulasToNodes;

    
    public OldClassicalProofTree(SignedFormulaNode aNode) {
        super(aNode);
        _closed = false;

        _fsmm = new FormulaSignMultimap();
        _PBCandidates = new SignedFormulaList();
        _signedFormulasToNodes = new HashMap();

        updateStructures(aNode);

    }

    private void updateSignedFormulasToNodesMap(SignedFormulaNode aNode) {
        _signedFormulasToNodes.put(aNode.getContent(), aNode);
    }

    private void updatePBCandidates(SignedFormulaNode aNode) {
        if (((SignedFormula) aNode.getContent()).getFormula() instanceof CompositeFormula) {
            _PBCandidates.add((SignedFormula) aNode.getContent());
        } else {
            aNode.setState(SignedFormulaNodeState.FULFILLED);
        }

    }

    private void updateMultimap(SignedFormulaNode aNode) {
        SignedFormula sf = (SignedFormula) aNode.getContent();
        _fsmm.put(sf.getFormula(), sf.getSign());
        if (_fsmm.get(sf.getFormula()).size() == 2) {
            _closed = true;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see proofTree.ProofTree#addLast(proofTree.Node)
     */
    public void addLast(Node aNode) {
        // TODO only adds if it is new -- do the same with Left and Right

        //        System.err.println ("addLast " + aNode.getContent() + " " +
        // _signedFormulasToNodes.keySet());

        //        if ((_signedFormulasToNodes.get(aNode.getContent())) == null ) {
        if (getNode((SignedFormula) aNode.getContent()) == null) {

            updateStructures(aNode);

            super.addLast(aNode);
        }
    }

    /**
     * @param aNode
     */
    protected void updateStructures(Node aNode) {
        updateMultimap((SignedFormulaNode) aNode);
        updatePBCandidates((SignedFormulaNode) aNode);
        updateSignedFormulasToNodesMap((SignedFormulaNode) aNode);
    }

    /* (non-Javadoc)
     * @see proofTree.ProofTree#setOtherStructures(proofTree.Node)
     */
    protected void setOtherStructures(ProofTree pt, Node aNode) {
        ((OldClassicalProofTree) pt).addPBCandidates(this.getPBCandidates());
    }
    
    /* (non-Javadoc)
     * @see proofTree.ProofTree#makeInstance(proofTree.Node)
     */
    protected ProofTree makeInstance(Node aNode) {
        return new OldClassicalProofTree((SignedFormulaNode) aNode);
    }

    /**
     * @param candidates
     */
    protected void addPBCandidates(SignedFormulaList candidates) {
        _PBCandidates.addAll(candidates);

    }


    //    x
    // TODO addLeft, addRight

    public boolean isClosed() {
        return _closed
                || (_left != null && ((OldClassicalProofTree) _left).isClosed() && ((OldClassicalProofTree) _right)
                        .isClosed());
    }

    public void setPBCandidates (SignedFormulaList sfl){
        _PBCandidates = sfl;
    }
    
    public void removeFromPBCandidates(SignedFormula sf) {
        _PBCandidates.remove(sf);
    }

    
    public void removeFromPBCandidates(int index) {
        _PBCandidates.remove(index);
    }

    public SignedFormulaList getPBCandidates() {
        return _PBCandidates;
    }

    public SignedFormulaNode getNode(SignedFormula formula) {

        SignedFormulaNode sfn = (SignedFormulaNode) _signedFormulasToNodes
                .get(formula);
        if (sfn != null) {
            return sfn;
        } else {
            if (_parent != null) {
                return ((OldClassicalProofTree) _parent).getNode(formula);
            }
        }

        return null;
    }

    public void removeFromPBCandidates(SignedFormula sf, SignedFormulaNodeState state) {
        SignedFormulaNode sfn = getNode(sf);
        sfn.setState(state);
        removeFromPBCandidates(sf);

        //        System.err.println ("after remove: "+ _PBCandidates);
    }
    
    /**
     * 
     */
    protected String getStatus() {
        
        return "closed? : "+isClosed() + System.getProperty("line.separator"); 

    }
    
    /**
     * @return Returns the _signedFormulasToNodes.
     */
    public Map getSignedFormulasToNodes() {
        return _signedFormulasToNodes;
    }
    
    /**
     * @return Returns the _fsmm.
     */
    public FormulaSignMultimap getFsmm() {
        return _fsmm;
    }
    
    public void eraseLeft() {
        _left = NullClosedProofTree.INSTANCE;

    }

    
}