/*
 * Created on 25/11/2004
 *
 */
package main.strategy.oldsimple;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import logic.formulas.Formula;
import logic.formulas.FormulaList;
import logic.signedFormulas.FormulaSign;
import logic.signedFormulas.SignedFormula;
import logic.signedFormulas.SignedFormulaList;
import logic.structures.FormulaSignedFormulaMultiMap;
import logic.structures.FormulaSubformulaReferencesMultiMap;
import logic.structures.SignSet;
import main.proofTree.Node;
import main.proofTree.ProofTree;
import main.proofTree.SignedFormulaNode;

/**
 * Class that represents proof trees where each formula that is part of a node
 * in the proof tree is mapped to the signed formulas that appear in a given
 * branch of the proof tree.
 * 
 * @author adolfo
 *  
 */
public class OldFormulaReferenceClassicalProofTree extends OldClassicalProofTree {

    FormulaSignedFormulaMultiMap _localReferences;

    // TIREI EM 26/01/2005 - no vi necessidade
    //    FormulaSignedFormulaMultiMap _localReferencesInCandidates;

    FormulaSubformulaReferencesMultiMap _subformulaLocalReferences;

    FormulaSubformulaReferencesMultiMap _subformulaLocalReferencesInCandidates;

    /**
     * @param aNode
     */
    public OldFormulaReferenceClassicalProofTree(SignedFormulaNode aNode) {
        super(aNode);
        _localReferences = new FormulaSignedFormulaMultiMap();
        // TIREI EM 26/01/2005 - no vi necessidade
        //        _localReferencesInCandidates = new FormulaSignedFormulaMultiMap();
        _subformulaLocalReferences = new FormulaSubformulaReferencesMultiMap();
        _subformulaLocalReferencesInCandidates = new FormulaSubformulaReferencesMultiMap();
        updateReferences(aNode);
    }

    /**
     * @param aNode
     */
    private void updateReferences(SignedFormulaNode aNode) {
        updateLocalReferences((SignedFormula) aNode.getContent(),
                _localReferences, _subformulaLocalReferences);
        // TIREI EM 26/01/2005 - no vi necessidade
        //        updateLocalReferences((SignedFormula) aNode.getContent(),
        //                _localReferencesInCandidates,
        //                _subformulaLocalReferencesInCandidates);
    }

    /*
     * (non-Javadoc)
     * 
     * @see proofTree.ClassicalProofTree#addLast(proofTree.Node)
     */
    public void addLast(Node aNode) {
        super.addLast(aNode);
        updateReferences((SignedFormulaNode) aNode);
    }

    /*
     * (non-Javadoc)
     * 
     * @see strategy.ClassicalProofTree#makeInstance(proofTree.Node)
     */
    protected ProofTree makeInstance(Node aNode) {
        return new OldFormulaReferenceClassicalProofTree((SignedFormulaNode) aNode);
    }

    /*
     * (non-Javadoc)
     * 
     * @see strategy.ClassicalProofTree#setOtherStructures(proofTree.ProofTree)
     */
    protected void setOtherStructures(ProofTree pt, Node aNode) {
        super.setOtherStructures(pt, aNode);
        ((OldFormulaReferenceClassicalProofTree) pt).addStructures(this);
        ((OldFormulaReferenceClassicalProofTree) pt).updateStructures(aNode);
        ((OldFormulaReferenceClassicalProofTree) pt)
                .updateReferences((SignedFormulaNode) aNode);
    }

    /**
     * @param tree
     */
    private void addStructures(OldFormulaReferenceClassicalProofTree tree) {

        List l = new ArrayList(tree._fsmm.keySet());
        for (int i = 0; i < l.size(); i++) {
            SignSet ss = tree._fsmm.get((Formula) l.get(i));

            Iterator it = ss.iterator();

            while (it.hasNext()) {
                this._fsmm.put((Formula) l.get(i), (FormulaSign) it.next());

            }
        }

        l = new ArrayList(tree._signedFormulasToNodes.keySet());
        for (int i = 0; i < l.size(); i++)
            this._signedFormulasToNodes.put(l.get(i),
                    tree._signedFormulasToNodes.get(l.get(i)));

    }

    
    //    /**
    //     * @param formula
    //     */
    //    private void updateLocalReferences(SignedFormula formula) {
    //        _localReferences.put(formula.getFormula(), formula);
    //        putAllSubformulasinLocalReferences(formula.getFormula(), formula);
    //    }

    private void updateLocalReferences(SignedFormula formula,
            FormulaSignedFormulaMultiMap references,
            FormulaSubformulaReferencesMultiMap subformulaReferences) {
        references.put(formula.getFormula(), formula);
        putAllSubformulasinLocalReferences(formula.getFormula(), formula,
                references, subformulaReferences);
    }

    //    /**
    //     * @param formula
    //     * @param sf
    //     */
    //    private void putAllSubformulasinLocalReferences(Formula formula,
    //            SignedFormula sf) {
    //        if (formula.getImmediateSubformulas().size() != 0) {
    //            for (int i = 0; i < formula.getImmediateSubformulas().size(); i++) {
    //                //
    //                // System.out.print((Formula) formula
    //                // .getImmediateSubformulas().get(i));
    //                // System.out.print(" " + sf + " ");
    //                // System.out.println(formula);
    //                _subformulaLocalReferences.put((Formula) formula
    //                        .getImmediateSubformulas().get(i), sf, formula);
    //                _localReferences.put((Formula) formula
    //                        .getImmediateSubformulas().get(i), sf);
    //                putAllSubformulasinLocalReferences((Formula) formula
    //                        .getImmediateSubformulas().get(i), sf);
    //            }
    //        }
    //    }

    private void putAllSubformulasinLocalReferences(Formula formula,
            SignedFormula sf, FormulaSignedFormulaMultiMap references,
            FormulaSubformulaReferencesMultiMap subformulaReferences) {
        if (formula.getImmediateSubformulas().size() != 0) {
            for (int i = 0; i < formula.getImmediateSubformulas().size(); i++) {
                //                
                //                System.out.print((Formula) formula
                //                        .getImmediateSubformulas().get(i));
                //                System.out.print(" " + sf + " ");
                //                System.out.println(formula);
                subformulaReferences.put((Formula) formula
                        .getImmediateSubformulas().get(i), sf, formula);
                references.put((Formula) formula.getImmediateSubformulas().get(
                        i), sf);
                putAllSubformulasinLocalReferences((Formula) formula
                        .getImmediateSubformulas().get(i), sf, references,
                        subformulaReferences);
            }
        }
    }

    /**
     * @param formula
     * @return
     */
    public SignedFormulaList getLocalReferences(Formula formula) {

        if (_localReferences.get(formula) != null)
            return (SignedFormulaList) _localReferences.get(formula);
        else
            return new SignedFormulaList();
    }

    /**
     * @param a
     * @param sf
     * @return
     */
    public FormulaList getSubformulaLocalReferences(Formula a, SignedFormula sf) {
        if (_subformulaLocalReferences.get(a, sf) != null)
            return _subformulaLocalReferences.get(a, sf);
        else
            return new FormulaList();
    }

    /**
     * @param formula
     * @return
     */
    public SignedFormulaList getParentReferences(Formula formula) {

        ProofTree current = this;
        SignedFormulaList sfl = new SignedFormulaList();
        do {
            sfl.addAll(((OldFormulaReferenceClassicalProofTree) current)
                    .getLocalReferences(formula));
            current = current.getParent();
        } while (current != null);

        return sfl;
    }

    /**
     * @param formula
     * @param sf
     * @return
     */
    public FormulaList getSubformulaParentReferences(Formula formula,
            SignedFormula sf) {

        ProofTree current = this;
        FormulaList fl = new FormulaList();
        do {
            fl.addAll(((OldFormulaReferenceClassicalProofTree) current)
                    .getSubformulaLocalReferences(formula, sf));
            current = current.getParent();
        } while (current != null);

        return fl;
    }

    // TIREI EM 26/01/2005 - no vi necessidade
    //    /**
    //     * @param topFormula
    //     * @return
    //     */
    //    public SignedFormulaList getLocalReferencesInCandidates(Formula formula)
    // {
    //        if (_localReferencesInCandidates.get(formula) != null)
    //            return (SignedFormulaList) _localReferencesInCandidates
    //                    .get(formula);
    //        else
    //            return new SignedFormulaList();
    //    }

}