/*
 * Created on 06/07/2005
 *
 */
package main.strategy.simple;

import java.util.LinkedList;

import logic.signedFormulas.SignedFormulaBuilder;
import main.proofTree.ProofTree;
import main.strategy.AbstractStrategy;
import main.strategy.ClassicalProofTree;
import main.tableau.Method;

/**
 * An abstract version of SimpleStrategy. To be used in the creation of new
 * strategies.
 * 
 * @author Adolfo Gustavo Serra Seca Neto
 *  
 */
public abstract class AbstractSimpleStrategy extends AbstractStrategy {

	/**
	 * Creates an AbstractSimpleStrategy
	 * 
	 * @param m
	 */
	public AbstractSimpleStrategy(Method m) {
		super(m);
		setCurrent(null);
	}

	/**
	 * @return the current proof tree in the subclass used by this strategy
	 */
	protected FormulaReferenceClassicalProofTree getThisCurrent() {
		return (FormulaReferenceClassicalProofTree) getCurrent();
	}

	/**
	 * @return the proof tree in the subclass used by this strategy
	 */
	protected FormulaReferenceClassicalProofTree getThisProofTree() {
		return (FormulaReferenceClassicalProofTree) getProofTree();
	}
	
	/**
	 * Applies as much linear rules as possible in the current branch.
	 * 
	 * @param sfb -
	 *            the formula builder
	 * @return true if has applied any rule
	 */
	protected final boolean applyLinearRules(SignedFormulaBuilder sfb) {
		boolean hasApplied;
		do {
			hasApplied = applyOnePremiseRules(getCurrent(), sfb);
			if (getCurrent().isClosed()) {
				break;
			}

			hasApplied = applyTOPandBOTTOMrules(getCurrent(), sfb)
					|| hasApplied;
			if (getCurrent().isClosed()) {
				break;
			}

			// first faster
			hasApplied = applyTwoPremiseRulesWithMap(
			//             hasApplied = applyTwoPremiseRulesWithoutMapCurrentDownThenUp(
					// hasApplied = applyTwoPremiseRulesWithoutMapBottomUp(
					getCurrent(), sfb) || hasApplied;
			if (getCurrent().isClosed()) {
				break;
			}

		} while (hasApplied);
		return hasApplied;
	}

	/**
	 * @param current
	 * @param sfb
	 * @return
	 */
	protected abstract boolean applyTwoPremiseRulesWithMap(ClassicalProofTree current, SignedFormulaBuilder sfb);

	/**
	 * @param current
	 * @param sfb
	 * @return
	 */
	protected abstract boolean applyTOPandBOTTOMrules(ClassicalProofTree current, SignedFormulaBuilder sfb); 

	/**
	 * @param current
	 * @param sfb
	 * @return
	 */
	protected abstract boolean applyOnePremiseRules(ClassicalProofTree current, SignedFormulaBuilder sfb);
	
	


}
