/*
 * Created on 25/11/2004
 *
 */
package main.strategy.simple;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import logic.formulas.Formula;
import logic.formulas.FormulaList;
import logic.signedFormulas.FormulaSign;
import logic.signedFormulas.SignedFormula;
import logic.signedFormulas.SignedFormulaList;
import logic.structures.FormulaSignedFormulaMultiMap;
import logic.structures.FormulaSubformulaReferencesMultiMap;
import logic.structures.SignSet;
import main.proofTree.Node;
import main.proofTree.ProofTree;
import main.proofTree.SignedFormulaNode;
import main.strategy.ClassicalProofTree;

/**
 * Class that represents proof trees where each formula that is part of a node
 * in the proof tree is mapped to the signed formulas that appear in a given
 * branch of the proof tree.
 * 
 * @author adolfo
 * 
 */
public class FormulaReferenceClassicalProofTree extends ClassicalProofTree {

    FormulaSignedFormulaMultiMap _localReferences;

    // TIREI EM 26/01/2005 - no vi necessidade
    // FormulaSignedFormulaMultiMap _localReferencesInCandidates;

    FormulaSubformulaReferencesMultiMap _subformulaLocalReferences;

    FormulaSubformulaReferencesMultiMap _subformulaLocalReferencesInCandidates;

    private boolean _closed = false;

    /**
     * @param aNode
     */
    public FormulaReferenceClassicalProofTree(SignedFormulaNode aNode) {
        super(aNode);
        _localReferences = new FormulaSignedFormulaMultiMap();
        // TIREI EM 26/01/2005 - no vi necessidade
        // _localReferencesInCandidates = new FormulaSignedFormulaMultiMap();
        _subformulaLocalReferences = new FormulaSubformulaReferencesMultiMap();
        _subformulaLocalReferencesInCandidates = new FormulaSubformulaReferencesMultiMap();
        updateReferences(aNode);
    }

    /**
     * @param aNode
     */
    private void updateReferences(SignedFormulaNode aNode) {
        updateLocalReferences((SignedFormula) aNode.getContent(),
                _localReferences, _subformulaLocalReferences);
        // TIREI EM 26/01/2005 - no vi necessidade
        // updateLocalReferences((SignedFormula) aNode.getContent(),
        // _localReferencesInCandidates,
        // _subformulaLocalReferencesInCandidates);
    }

    protected void updateMultimap(SignedFormulaNode aNode) {
        SignedFormula sf = (SignedFormula) aNode.getContent();
        getFsmm().put(sf.getFormula(), sf.getSign());
        if (getFsmm().get(sf.getFormula()).size() == 2) {
            //            
            // setLocallyClosed(true);
            _closed = true;
            // System.err.println(sf);
            tryToCloseParent();
            //            
        }
    }

    // TODO  repetido!
    protected void updateStructures(Node aNode) {
        updateMultimap((SignedFormulaNode) aNode);
        updatePBCandidates((SignedFormulaNode) aNode);
        updateSignedFormulasToNodesMap((SignedFormulaNode) aNode);
    }

    public void tryToCloseParent() {
        // System.err.println (this.getBranchNumber()+ " fechou. Subindo para
        // talvez fechar");

        if ((_parent != null) && (this == _parent.getRight())
                && !(((FormulaReferenceClassicalProofTree) _parent)._closed)) {
            ((FormulaReferenceClassicalProofTree) _parent)._closed = true;
            // System.err.println (" Fechou "+ _parent.getBranchNumber());

            FormulaReferenceClassicalProofTree _previous = (FormulaReferenceClassicalProofTree) _parent;
            FormulaReferenceClassicalProofTree _current = ((FormulaReferenceClassicalProofTree) _parent
                    .getParent());
            while (_current != null && !_current._closed) {
                // System.err.print(_current.getBranchNumber() + " ");
                if (_previous == (FormulaReferenceClassicalProofTree) _current
                        .getRight()) {
                    ((FormulaReferenceClassicalProofTree) _current)._closed = true;
                    _previous = _current;
                    _current = (FormulaReferenceClassicalProofTree) _current
                            .getParent();
                } else {
                    // System.err.println (" No fechou "+
                    // _current.getBranchNumber());
                    break;
                }
            }

        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see strategy.ClassicalProofTree#isClosed()
     */
    public boolean isClosed() {
        // TODO REPETIDO
        // if (!_closed){
        // _closed= (getLeft() != null
        // && ((FormulaReferenceClassicalProofTree) getLeft()).isClosed() &&
        // ((FormulaReferenceClassicalProofTree) getRight())
        // .isClosed());
        // }
        return _closed;
        // return super.isLocallyClosed()
        // || (getLeft() != null
        // && ((ClassicalProofTree) getLeft()).isClosed() &&
        // ((ClassicalProofTree) getRight())
        // .isClosed());
    }

    /*
     * (non-Javadoc)
     * 
     * @see proofTree.ClassicalProofTree#addLast(proofTree.Node)
     */
    public void addLast(Node aNode) {
        super.addLast(aNode);
        updateReferences((SignedFormulaNode) aNode);
    }

    /*
     * (non-Javadoc)
     * 
     * @see strategy.ClassicalProofTree#makeInstance(proofTree.Node)
     */
    protected ProofTree makeInstance(Node aNode) {
        return new FormulaReferenceClassicalProofTree((SignedFormulaNode) aNode);
    }

    /*
     * (non-Javadoc)
     * 
     * @see strategy.ClassicalProofTree#setOtherStructures(proofTree.ProofTree)
     */
    protected void setOtherStructures(ProofTree pt, Node aNode) {
        super.setOtherStructures(pt, aNode);
        ((FormulaReferenceClassicalProofTree) pt).addStructures(this);
        ((FormulaReferenceClassicalProofTree) pt).updateStructures(aNode);
        ((FormulaReferenceClassicalProofTree) pt)
                .updateReferences((SignedFormulaNode) aNode);
    }

    /**
     * @param tree
     */
    private void addStructures(FormulaReferenceClassicalProofTree tree) {

        List l = new ArrayList(tree.getFsmm().keySet());
        for (int i = 0; i < l.size(); i++) {
            SignSet ss = tree.getFsmm().get((Formula) l.get(i));

            Iterator it = ss.iterator();

            while (it.hasNext()) {
                this.getFsmm().put((Formula) l.get(i), (FormulaSign) it.next());

            }
        }

        l = new ArrayList(tree.getSignedFormulasToNodes().keySet());
        for (int i = 0; i < l.size(); i++)
            this.getSignedFormulasToNodes().put(l.get(i),
                    tree.getSignedFormulasToNodes().get(l.get(i)));

    }

    // /**
    // * @param formula
    // */
    // private void updateLocalReferences(SignedFormula formula) {
    // _localReferences.put(formula.getFormula(), formula);
    // putAllSubformulasinLocalReferences(formula.getFormula(), formula);
    // }

    private void updateLocalReferences(SignedFormula formula,
            FormulaSignedFormulaMultiMap references,
            FormulaSubformulaReferencesMultiMap subformulaReferences) {
        references.put(formula.getFormula(), formula);
        putAllSubformulasinLocalReferences(formula.getFormula(), formula,
                references, subformulaReferences);
    }

    // /**
    // * @param formula
    // * @param sf
    // */
    // private void putAllSubformulasinLocalReferences(Formula formula,
    // SignedFormula sf) {
    // if (formula.getImmediateSubformulas().size() != 0) {
    // for (int i = 0; i < formula.getImmediateSubformulas().size(); i++) {
    // //
    // // System.out.print((Formula) formula
    // // .getImmediateSubformulas().get(i));
    // // System.out.print(" " + sf + " ");
    // // System.out.println(formula);
    // _subformulaLocalReferences.put((Formula) formula
    // .getImmediateSubformulas().get(i), sf, formula);
    // _localReferences.put((Formula) formula
    // .getImmediateSubformulas().get(i), sf);
    // putAllSubformulasinLocalReferences((Formula) formula
    // .getImmediateSubformulas().get(i), sf);
    // }
    // }
    // }

    private void putAllSubformulasinLocalReferences(Formula formula,
            SignedFormula sf, FormulaSignedFormulaMultiMap references,
            FormulaSubformulaReferencesMultiMap subformulaReferences) {
        if (formula.getImmediateSubformulas().size() != 0) {
            for (int i = 0; i < formula.getImmediateSubformulas().size(); i++) {
                //                
                // System.out.print((Formula) formula
                // .getImmediateSubformulas().get(i));
                // System.out.print(" " + sf + " ");
                // System.out.println(formula);
                subformulaReferences.put((Formula) formula
                        .getImmediateSubformulas().get(i), sf, formula);
                references.put((Formula) formula.getImmediateSubformulas().get(
                        i), sf);
                putAllSubformulasinLocalReferences((Formula) formula
                        .getImmediateSubformulas().get(i), sf, references,
                        subformulaReferences);
            }
        }
    }

    /**
     * @param formula
     * @return
     */
    public SignedFormulaList getLocalReferences(Formula formula) {

        if (_localReferences.get(formula) != null)
            return (SignedFormulaList) _localReferences.get(formula);
        else
            return new SignedFormulaList();
    }

    /**
     * @param a
     * @param sf
     * @return
     */
    public FormulaList getSubformulaLocalReferences(Formula a, SignedFormula sf) {
        if (_subformulaLocalReferences.get(a, sf) != null)
            return _subformulaLocalReferences.get(a, sf);
        else
            return new FormulaList();
    }

    /**
     * @param formula
     * @return
     */
    public SignedFormulaList getParentReferences(Formula formula) {

        ProofTree current = this;
        SignedFormulaList sfl = new SignedFormulaList();
        do {
            sfl.addAll(((FormulaReferenceClassicalProofTree) current)
                    .getLocalReferences(formula));
            current = current.getParent();
        } while (current != null);

        return sfl;
    }

    /**
     * @param formula
     * @param sf
     * @return
     */
    public FormulaList getSubformulaParentReferences(Formula formula,
            SignedFormula sf) {

        ProofTree current = this;
        FormulaList fl = new FormulaList();
        do {
            fl.addAll(((FormulaReferenceClassicalProofTree) current)
                    .getSubformulaLocalReferences(formula, sf));
            current = current.getParent();
        } while (current != null);

        return fl;
    }

    // TIREI EM 26/01/2005 - no vi necessidade
    // /**
    // * @param topFormula
    // * @return
    // */
    // public SignedFormulaList getLocalReferencesInCandidates(Formula formula)
    // {
    // if (_localReferencesInCandidates.get(formula) != null)
    // return (SignedFormulaList) _localReferencesInCandidates
    // .get(formula);
    // else
    // return new SignedFormulaList();
    // }

}