/*
 * Created on 23/03/2005
 *
 */
package problemGenerator;

import java.io.FileWriter;

import logic.problem.Problem;
import util.StringUtil;

/**
 * Abstract class for generators of families of problems.
 * 
 * @author Adolfo Gustavo Serra Seca Neto
 * 
 * 
 */
public abstract class ProblemGenerator {
    private int maxProblemSize = 1;

    private String familyName;

    private static String nameSeparator = "_";

    protected ProblemGenerator(String familyName) {
        this.familyName = familyName;
    }

    /**
     * @return name of the family of problems
     */
    public String getFamilyName() {
        return familyName;
    }

    /**
     * @return Returns the maxProblemSize.
     */
    public int getMaxProblemSize() {
        return maxProblemSize;
    }

    /**
     * @return separator for names
     */
    public static String getNameSeparator() {
        return nameSeparator;
    }

    /**
     * generates an instance of a problem in Wagner Dias Format
     * 
     * @param index
     * @return
     */
    public Problem generate(int index) {
        Problem p = new Problem("Wagner Dias's format");
        p.setName(getFamilyName() + getNameSeparator()
                + StringUtil.zeroes(getMaxProblemSize(), index) + index);
        return generate(p, index);
    }

    /** generates an instance of a problem.
     * @param p
     * @param index
     * @return
     */
    protected abstract Problem generate(Problem p, int index);

    /**
     * Saves a problem in a file
     * 
     * @param path
     * @param index
     * @param fileTerminator
     */
    public void saveFile(String path, int index, String fileTerminator) {
        Problem p = generate(index);
        p.setFilename(path + System.getProperty("file.separator") + p.getName()
                + fileTerminator);
        save(p);
    }

    /**
     * Saves a sequence of problems in files.
     * 
     * @param path
     * @param begin
     * @param end
     * @param fileTerminator
     */
    public void saveSequence(String path, int begin, int end,
            String fileTerminator) {
        maxProblemSize = end;
        for (int i = begin; i <= end; i++) {
            saveFile(path, i, fileTerminator);
        }
    }
    /** saves a problem in a file
     * @param p
     */
    private void save(Problem p) {
        try {
            FileWriter f = new FileWriter(p.getFilename());
            f.write(p.getFormulas().get(0).toString());
            f.close();
        } catch (Exception e) {
            System.err.println("ProblemGenerator: Unable to generate file "
                    + p.getFilename() + " for problem " + p.getName());
        }
    }
}
