/*
 * Created on 31/03/2005
 *
 */
package prooftreeviewer;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;

import proofTree.Node;
import proofTree.ProofTree;

public class ProofTreeCanvas extends Canvas {
    Image buffImage;

    Graphics offscreen;

    boolean initDone = false;

    private ProofTree proofTree;

    private Dimension dim;

    private String baseWidth = "1234567890123";

    ProofTreeCanvas(ProofTree pt, Dimension dim) {
        this.proofTree = pt;
        this.dim = dim;
    }

    public void paint(Graphics g) {
        if (!initDone)
            initpaint(g);
        else
            g.drawImage(buffImage, 0, 0, this);
    }

    public void update(Graphics g) {
        g.drawImage(buffImage, 0, 0, this);
    }

    public void initpaint(Graphics g) {
        try {
            int maxX = (int) dim.getWidth();
            int maxY = (int) dim.getHeight();

            buffImage = this.createImage(maxX, maxY);
            offscreen = buffImage.getGraphics();
            offscreen.setColor(Color.white);
            offscreen.fillRect(0, 0, maxX, maxY);
            offscreen.setColor(Color.black);
            offscreen.setFont(new Font("Courier", Font.ITALIC, 10));
            FontMetrics metrics = offscreen.getFontMetrics();

            int beginY = 20;
            int y = beginY;
            int fh = metrics.getHeight();

            showProofTree(offscreen, maxX, beginY);

            // for (int i = 0; i < ((maxY - beginY) / fh); i++) {
            //
            // int stringSize = metrics.stringWidth(baseWidth);
            // int x = (int) (maxX - stringSize - (Math.random() * (maxX -
            // stringSize)));
            // offscreen.drawString(i + "(" + x + "," + y + ")", x, y);
            // y += fh;
            // }
            initDone = true;
            g.drawImage(buffImage, 0, 0, this);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void showProofTree(Graphics screen, int maxX, int beginY) {
        FontMetrics metrics = screen.getFontMetrics();
        int fh = metrics.getHeight();
        showProofTree(proofTree, screen, "", 0, 0, maxX, beginY, fh, 0);

    }

    private void showProofTree(ProofTree proofTree, Graphics screen,
            String prefix, int spaces, int xi, int xf, int y, int fh, int level) {

        String result = "";

        // result = spaces(spaces) + result + proofTree.getStatus();
        Node current = proofTree.getRoot();

        do {

            result = spaces(spaces) + prefix + (spaces > 0 ? spaces + ":" : "")
                    + current.getContent().toString();
            // + current.getContent().hashCode();
            ;

            // screen.drawString(result, x, y);
            // y += fh;
            screen.setColor(Color.black);
            screen.fillRect(((xi + xf) / 2)-1, y-1, 2,2);
//            screen.drawLine((xi + xf) / 2, y, (xi + xf) / 2, y);
            // screen.fillOval(((xi+xf)/2)-2,y-2,2,2);
            System.out.println(level + " " + current.getContent().toString()
                    + " at " + ((xi + xf) / 2) + "," + y);
            // screen.drawOval(((xi+xf)/2)-2,y-2,2,2);
            // y+=4;
            y += 2;
            current = current.getNext();
        } while (current != null);

        if (proofTree.getLeft() != null) {
            screen.setColor(Color.blue);
            screen.drawLine( (xi + xf) / 2, y-2, (xi+((xi+xf)/2))/2,y);
            screen.drawLine( (xi + xf) / 2, y-2, (xf+((xi+xf)/2))/2,y);
            screen.setColor(Color.black);
            showProofTree(proofTree.getLeft(), screen, "l", spaces + 1, xi,
                    (xi + xf) / 2, y, fh, level + 1);
            showProofTree(proofTree.getRight(), screen, "r", spaces + 1,
                    (xi + xf) / 2, xf, y, fh, level + 1);
        }

    }

    /**
     * @param i
     * @return a number of spaces
     */
    private String spaces(int i) {
        String result = "";

        for (int j = 0; j < i; j++) {
            result = result + " ";
        }
        return result;
    }

}
