/*
 * Created on 01/02/2005
 *
 */
package util;

/**
 * Helps to perform garbage collection and to get memory usage measures.
 * 
 * @author Adolfo Gustavo Serra Seca neto
 * 
 */
public class MemoryManager {

    private static final Runtime s_runtime = Runtime.getRuntime();

    /**
     * Tries hardly to perform a garbage collection.
     * 
     * @throws Exception
     */
    public static void runGC() throws Exception {
        for (int r = 0; r < 4; ++r)
            _runGC();
    }

    private static void _runGC() throws Exception {
        long usedMem1 = usedMemory(), usedMem2 = Long.MAX_VALUE;
        for (int i = 0; (usedMem1 < usedMem2) && (i < 500); ++i) {
            s_runtime.runFinalization();
            s_runtime.gc();
            Thread.yield();
            usedMem2 = usedMem1;
            usedMem1 = usedMemory();
        }
    }

    /**
     * Returns amount of memory used.
     * 
     * @return memory used in bytes
     */
    public static long usedMemory() {
        return s_runtime.totalMemory() - s_runtime.freeMemory();
    }

}
